/*!
 * nextjs-chunk-upload-action
 *
 * Uploading large files with chunking using server action in Next.js
 *
 * [GitHub]: https://github.com/a179346/nextjs-chunk-upload-action
 * [npm]: https://www.npmjs.com/package/nextjs-chunk-upload-action
 * [API Reference]: https://github.com/a179346/nextjs-chunk-upload-action/blob/main/docs/api-reference.md
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var ChunkUploader = /** @class */ (function () {
    function ChunkUploader(options) {
        this._validateOptions(options);
        this._status = 'pending';
        this._position = 0;
        this._file = options.file;
        this._onChunkUpload = options.onChunkUpload;
        this._chunkBytes = options.chunkBytes || 1024 * 1024 * 5;
        this._metadata = options.metadata;
        this._retryDelays = options.retryDelays || [1000, 2000, 4000, 8000];
        this._onChunkComplete = options.onChunkComplete;
        this._onSuccess = options.onSuccess;
        this._onError = options.onError;
        this._onPaused = options.onPaused;
        this._onAborted = options.onAborted;
        this._onStatusChange = options.onStatusChange;
        this._statusChangedEventListeners = {};
        if (this._onStatusChange)
            this._onStatusChange(undefined, this.status);
    }
    Object.defineProperty(ChunkUploader.prototype, "status", {
        /*************
         * Public
         *************/
        get: function () {
            return this._status;
        },
        set: function (value) {
            var oldValue = this._status;
            if (oldValue === value)
                return;
            this._status = value;
            var listenerSet = this._statusChangedEventListeners[value];
            if (listenerSet)
                listenerSet.forEach(function (listener) { return listener(); });
            if (this._onStatusChange)
                this._onStatusChange(oldValue, value);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ChunkUploader.prototype, "bytesUploaded", {
        get: function () {
            return this._position;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ChunkUploader.prototype, "error", {
        get: function () {
            return this._error;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ChunkUploader.prototype, "file", {
        get: function () {
            return this._file;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Start the upload process.
     * returns `false` if the status is not `pending`.
     *
     * status: `pending` -> `uploading` -> `complete` or `error`
     */
    ChunkUploader.prototype.start = function () {
        if (!this.canStart)
            return false;
        this.status = 'uploading';
        this._error = undefined;
        this._startUploadFromCurrentPosition().catch(function () { });
        return true;
    };
    Object.defineProperty(ChunkUploader.prototype, "canStart", {
        get: function () {
            return this.status === 'pending';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Pause the upload process.
     * returns `false` if the status is not `uploading`.
     *
     * Note that the status at the end could be `complete` if the last chunk is being uploaded when the function is called.
     *
     * status: `uploading` -> `pausing` -> `paused` or `complete`
     */
    ChunkUploader.prototype.pause = function () {
        if (!this.canPause)
            return false;
        this.status = 'pausing';
        return true;
    };
    Object.defineProperty(ChunkUploader.prototype, "canPause", {
        get: function () {
            return this.status === 'uploading';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Resume the upload process.
     * returns `false` if the status is not `paused` or `error`.
     *
     * status: `paused` or `error` -> `uploading` -> `complete` or `error`
     */
    ChunkUploader.prototype.resume = function () {
        if (!this.canResume)
            return false;
        this.status = 'uploading';
        this._error = undefined;
        this._startUploadFromCurrentPosition().catch(function () { });
        return true;
    };
    Object.defineProperty(ChunkUploader.prototype, "canResume", {
        get: function () {
            return this.status === 'paused' || this.status === 'error';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Abort the upload process.
     * returns `false` if the status is not `paused` or `error`.
     *
     * status: `paused` or `error` -> `aborted`
     */
    ChunkUploader.prototype.abort = function () {
        if (!this.canAbort)
            return false;
        this.status = 'aborted';
        if (this._onAborted)
            this._onAborted(this._metadata);
        return true;
    };
    Object.defineProperty(ChunkUploader.prototype, "canAbort", {
        get: function () {
            return this.status === 'paused' || this.status === 'error';
        },
        enumerable: false,
        configurable: true
    });
    ChunkUploader.prototype._startUploadFromCurrentPosition = function () {
        return __awaiter(this, void 0, void 0, function () {
            var isLastChunkUploaded;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        isLastChunkUploaded = false;
                        _a.label = 1;
                    case 1:
                        if (!!isLastChunkUploaded) return [3 /*break*/, 3];
                        return [4 /*yield*/, this._uploadNextChunk()];
                    case 2:
                        isLastChunkUploaded = _a.sent();
                        if (this.status === 'pausing') {
                            this.status = 'paused';
                            if (this._onPaused)
                                this._onPaused();
                            return [2 /*return*/];
                        }
                        return [3 /*break*/, 1];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    ChunkUploader.prototype._uploadNextChunk = function () {
        return __awaiter(this, void 0, void 0, function () {
            var isLastChunk, endPosition, blob, retry, chunkFormData, error_1, isPausd;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        isLastChunk = this._position + this._chunkBytes >= this._file.size;
                        endPosition = isLastChunk ? this._file.size : this._position + this._chunkBytes;
                        return [4 /*yield*/, this._file.slice(this._position, endPosition)];
                    case 1:
                        blob = _a.sent();
                        retry = 0;
                        _a.label = 2;
                    case 2:
                        if (!(retry <= this._retryDelays.length)) return [3 /*break*/, 10];
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 9]);
                        chunkFormData = new FormData();
                        chunkFormData.set('blob', blob);
                        chunkFormData.set('offset', this._position.toString());
                        chunkFormData.set('length', blob.size.toString());
                        chunkFormData.set('retry', retry.toString());
                        chunkFormData.set('total', this._file.size.toString());
                        chunkFormData.set('isLastChunk', isLastChunk ? 'true' : 'false');
                        return [4 /*yield*/, this._onChunkUpload(chunkFormData, this._metadata)];
                    case 4:
                        _a.sent();
                        return [3 /*break*/, 10];
                    case 5:
                        error_1 = _a.sent();
                        if (this.status === 'pausing')
                            return [2 /*return*/, false];
                        if (!(retry < this._retryDelays.length)) return [3 /*break*/, 7];
                        return [4 /*yield*/, this._waitForRetry(this._retryDelays[retry])];
                    case 6:
                        isPausd = _a.sent();
                        if (isPausd)
                            return [2 /*return*/, false];
                        return [3 /*break*/, 8];
                    case 7:
                        this.status = 'error';
                        this._error = error_1;
                        if (this._onError)
                            this._onError(error_1);
                        throw error_1;
                    case 8: return [3 /*break*/, 9];
                    case 9:
                        retry += 1;
                        return [3 /*break*/, 2];
                    case 10:
                        this._position = endPosition;
                        if (isLastChunk)
                            this.status = 'complete';
                        if (this._onChunkComplete)
                            this._onChunkComplete(endPosition, this._file.size);
                        if (isLastChunk && this._onSuccess)
                            this._onSuccess();
                        return [2 /*return*/, isLastChunk];
                }
            });
        });
    };
    ChunkUploader.prototype._validateOptions = function (options) {
        if (!options.file)
            throw new Error('File is required');
        if (typeof options.file.size !== 'number')
            throw new Error('File size must be a number');
        if (typeof options.file.slice !== 'function')
            throw new Error('File slice must be a function');
        if (typeof options.onChunkUpload !== 'function')
            throw new Error('onChunkUpload must be a function');
        if (options.chunkBytes !== undefined) {
            if (typeof options.chunkBytes !== 'number')
                throw new Error('Chunk size must be a number');
            if (!Number.isInteger(options.chunkBytes))
                throw new Error('Chunk size must be an integer');
            if (options.chunkBytes <= 0)
                throw new Error('Chunk size must be greater than 0');
        }
        if (options.retryDelays !== undefined) {
            if (!Array.isArray(options.retryDelays))
                throw new Error('Retry delays must be an array');
            for (var _i = 0, _a = options.retryDelays; _i < _a.length; _i++) {
                var delay = _a[_i];
                if (typeof delay !== 'number')
                    throw new Error('Retry delay must be a number');
                if (!Number.isInteger(delay))
                    throw new Error('Retry delay must be an integer');
                if (delay < 0)
                    throw new Error('Retry delay must be greater than or equal to 0');
            }
        }
        if (options.onChunkComplete !== undefined) {
            if (typeof options.onChunkComplete !== 'function')
                throw new Error('onChunkComplete must be a function');
        }
        if (options.onSuccess !== undefined) {
            if (typeof options.onSuccess !== 'function')
                throw new Error('onSuccess must be a function');
        }
        if (options.onError !== undefined) {
            if (typeof options.onError !== 'function')
                throw new Error('onError must be a function');
        }
        if (options.onPaused !== undefined) {
            if (typeof options.onPaused !== 'function')
                throw new Error('onPaused must be a function');
        }
        if (options.onAborted !== undefined) {
            if (typeof options.onAborted !== 'function')
                throw new Error('onAborted must be a function');
        }
        if (options.onStatusChange !== undefined) {
            if (typeof options.onStatusChange !== 'function')
                throw new Error('onStatusChange must be a function');
        }
    };
    ChunkUploader.prototype._waitForRetry = function (ms) {
        var _this = this;
        return new Promise(function (resolve) {
            var isResolved = false;
            var handleTimeout = function () {
                if (isResolved)
                    return;
                isResolved = true;
                _this._removeStatusChangedEventListener('pausing', handlePause);
                clearTimeout(timeoutId);
                resolve(false);
            };
            var handlePause = function () {
                if (isResolved)
                    return;
                isResolved = true;
                _this._removeStatusChangedEventListener('pausing', handlePause);
                clearTimeout(timeoutId);
                resolve(true);
            };
            var timeoutId = setTimeout(handleTimeout, ms);
            _this._addStatusChangedEventListener('pausing', handlePause);
        });
    };
    ChunkUploader.prototype._addStatusChangedEventListener = function (status, listener) {
        var listenerSet = this._statusChangedEventListeners[status] || new Set();
        if (!this._statusChangedEventListeners[status])
            this._statusChangedEventListeners[status] = listenerSet;
        listenerSet.add(listener);
    };
    ChunkUploader.prototype._removeStatusChangedEventListener = function (status, listener) {
        var listenerSet = this._statusChangedEventListeners[status] || new Set();
        if (!this._statusChangedEventListeners[status])
            this._statusChangedEventListeners[status] = listenerSet;
        listenerSet.delete(listener);
    };
    return ChunkUploader;
}());
export { ChunkUploader };
//# sourceMappingURL=index.js.map